@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::C;
-$Time::C::VERSION = '0.012';
+$Time::C::VERSION = '0.013';
 # ABSTRACT: Convenient time manipulation.
 
 use overload (
@@ -14,6 +14,7 @@ use Carp qw/ croak /;
 use Function::Parameters qw/ :strict /;
 use Time::C::Sentinel;
 use Time::D;
+use Time::P ();
 use Time::Moment;
 use Time::Piece ();
 use Time::Zone::Olson;
@@ -36,6 +37,103 @@ method new ($c: $year = undef, $month = 1, $day = 1, $hour = 0, $minute = 0, $se
 }
 
 
+method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$month =, :$wday =, :$week =, :$yday =, :$year =, :$tz =, :$offset =) {
+
+    # Alright, time to try and construct a Time::C object with what we have
+    # We'll start with the easiest one: epoch
+    # Then go on to creating it from the date bits, and the time bits
+
+    my $t;
+
+    if (defined $epoch) {
+        $t = Time::C->gmtime($epoch);
+
+        if (defined $tz) {
+            $t->tz = $tz;
+        } elsif (defined $offset) {
+            $t->offset = $offset;
+        }
+
+        return $t;
+    } elsif (defined $year) { # We have a year at least...
+        if (defined $month) {
+            if (defined $mday) {
+                $t = Time::C->new($year, $month, $mday);
+            } else {
+                $t = Time::C->new($year, $month);
+            }
+        } elsif (defined $week) {
+            $t = Time::C->new($year)->week($week)->day_of_week(1);
+            if (defined $wday) { $t->day_of_week = $wday; }
+        } elsif (defined $yday) {
+            $t = Time::C->new($year)->day($yday);
+        } else { # we have neither month, week, or day of year!
+            $t = Time::C->new($year);
+        }
+
+        # Now add the time bits on top...
+        if (defined $hour) { $t->hour = $hour; }
+        if (defined $minute) { $t->minute = $minute; }
+        if (defined $second) { $t->second = $second; }
+    } else {
+        # If we don't have a year, let's use the current year
+        $year = Time::C->now($tz)->tz('UTC', 1)->year;
+        if (defined $month) {
+            if (defined $mday) {
+                $t = Time::C->new($year, $month, $mday);
+            } else {
+                $t = Time::C->new($year, $month);
+            }
+
+            # Now add the time bits on top...
+            if (defined $hour) { $t->hour = $hour; }
+            if (defined $minute) { $t->minute = $minute; }
+            if (defined $second) { $t->second = $second; }
+        } elsif (defined $week) {
+            $t = Time::C->new($year)->week($week)->day_of_week(1);
+            if (defined $wday) { $t->day_of_week = $wday; }
+
+            # Now add the time bits on top...
+            if (defined $hour) { $t->hour = $hour; }
+            if (defined $minute) { $t->minute = $minute; }
+            if (defined $second) { $t->second = $second; }
+        } elsif (defined $yday) {
+            $t = Time::C->new($year)->day($yday);
+
+            # Now add the time bits on top...
+            if (defined $hour) { $t->hour = $hour; }
+            if (defined $minute) { $t->minute = $minute; }
+            if (defined $second) { $t->second = $second; }
+        } else {
+            # We have neither year, month, week, or day of year ...
+            # So let's just make a time for today's date
+            $t = Time::C->now($tz)->second_of_day(0)->tz('UTC', 1);
+
+            croak "Could not mktime: No date specified and no time given."
+              if not defined $hour and not defined $minute and not defined $second;
+
+            # And add the time bits on top...
+            # - if hour not defined, use current hour
+            # - if hour and minute not defined, use current minute
+            if (defined $hour) { $t->hour = $hour; } else { $t->hour = Time::C->now($tz)->tz('UTC', 1)->hour; }
+            if (defined $minute) { $t->minute = $minute; } elsif (not defined $hour) { $t->second_of_day = Time::C->now($tz)->tz('UTC', 1)->second(0)->second_of_day; }
+            if (defined $second) { $t->second = $second; }
+        }
+    }
+
+    # And last, adjust for timezone bits
+
+    if (defined $tz) {
+        $t = $t->tz($tz, 1);
+    } elsif (defined $offset) {
+        $t->tm = $t->tm->with_offset_same_local($offset);
+        $t->offset = $offset;
+    }
+
+    return $t;
+}
+
+
 method localtime ($c: $epoch, $tz = $ENV{TZ}) {
     $tz = 'UTC' unless defined $tz;
     _verify_tz($tz);
@@ -52,17 +150,15 @@ method now ($c: $tz = $ENV{TZ}) { $c->localtime( time, $tz ); }
 method now_utc ($c:) { $c->localtime( time, 'UTC' ); }
 
 
-method from_string ($c: $str, $format = undef, $expected_tz = 'UTC') {
-    my @p = _parse($str, $format, $expected_tz);
+method from_string ($c: $str, :$format = undef, :$locale = 'C', :$strict = 1, :$tz = 'UTC') {
+    my @p = _parse($str, $format, $tz, locale => $locale, strict => $strict);
 
     bless {epoch => $p[0], tz => $p[1]}, $c;
 }
 
 
-method strptime ($c: $str, $format) {
-    my @p = _parse($str, $format, 'UTC');
-
-    bless {epoch => $p[0], tz => $p[1]}, $c;
+method strptime ($c: $str, $format, :$locale = 'C', :$strict = 1) {
+    return Time::P::strptime(@_);
 }
 
 fun _verify_tz ($tz) {
@@ -112,19 +208,17 @@ fun _get_tz ($offset) {
     return sprintf "%s%02s:%02s", $sign, $hour, $min;
 }
 
-fun _parse ($str, $format = undef, $tz = 'UTC') {
-    my $tm;
-
+fun _parse ($str, $format = undef, $tz = 'UTC', :$locale = 'C', :$strict = 1) {
     if (defined $format) {
-        my $tp = eval { local $ENV{TZ} = $tz; Time::Piece->localtime->strptime($str, $format); };
-        croak sprintf "Could not parse %s using %s.", $str, $format if not defined $tp;
+        my $t = eval { Time::P::strptime($str, $format, locale => $locale, strict => $strict) };
+        return ($t->epoch, $t->tz) if defined $t;
 
-        $tm = Time::Moment->from_object($tp);
-    } else {
-        $tm = eval { Time::Moment->from_string($str); };
-        croak sprintf "Could not parse %s.", $str if not defined $tm;
+        croak sprintf "Could not parse %s using %s: %s", $str, $format, $@;
     }
 
+    my $tm = eval { Time::Moment->from_string($str); };
+    croak sprintf "Could not parse %s.", $str if not defined $tm;
+
     my $epoch = $tm->epoch;
     my $offset = $tm->offset;
 
@@ -227,11 +321,11 @@ method tm ($t: $new_tm = undef) :lvalue {
 }
 
 
-method string ($t: $format = undef, $new_str = undef) :lvalue {
+method string ($t: $new_str = undef, :$format = undef, :$locale = 'C', :$strict = 1) :lvalue {
     $t->{tz} = 'UTC' if not defined $t->{tz};
 
     my $setter = sub {
-        @{$t}{epoch,tz} = _parse($_[0], $format, $t->{tz});
+        @{$t}{epoch,tz} = _parse($_[0], $format, $t->{tz}, locale => $locale, strict => $strict);
 
         return $t if defined $new_str;
         return $_[0];
@@ -242,6 +336,7 @@ method string ($t: $format = undef, $new_str = undef) :lvalue {
     my $str;
     if (defined $format) {
         local $ENV{TZ} = $t->{tz};
+        local $ENV{LC_ALL} = $locale;
         my $tp = Time::Piece->localtime($t->{epoch});
         $str = $tp->strftime($format);
     } else {
@@ -482,7 +577,7 @@ Time::C - Convenient time manipulation.
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 SYNOPSIS
 
@@ -557,6 +652,79 @@ This is the timezone specification such as C<Europe/Stockholm> or C<UTC>. If not
 
 =back
 
+=head2 mktime
+
+  my $t = Time::C->mktime(
+    epoch => epoch,
+    second => $second,
+    minute => $minute,
+    hour => $hour,
+    mday => $mday,
+    month => $month,
+    wday => $wday,
+    week => $week,
+    yday => $yday,
+    year => $year,
+    tz => $tz,
+    offset => $offset,
+  );
+
+Creates a Time::C object for the specified arguments. All the arguments are optional, as long as there is at least one way to specify some kind of time with them.
+
+If there is no date specified, it will default to today's date. If there is no timezone or offset specified, it will default to UTC. If there is a date, but no time specified, it will default to midnight.
+
+=over
+
+=item C<< epoch => $epoch >>
+
+If the C<$epoch> is specified, it overrides all the other options but C<$tz> and C<$offset>, and this basically becomes a call to C<< Time::C->gmtime($epoch); >>, applying the C<$tz> or C<$offset> afterwards.
+
+=item C<< second => $second >>
+
+C<$second> sets the second of the day/hour/minute, depending on what other options were specified.
+
+=item C<< minute => $minute >>
+
+C<$minute> sets the minute of the day/hour, depending on what other options were specified.
+
+=item C<< hour => $hour >>
+
+C<$hout> sets the hour of the day.
+
+=item C<< mday => $mday >>
+
+C<$mday> sets the day of the month, if a C<$month> was specified.
+
+=item C<< month => $month >>
+
+C<$month> sets the month of the year. If no C<$mday> is specified, it will default to the C<1st> day of the month.
+
+=item C<< wday => $wday >>
+
+C<$wday> sets the day of the week, if a C<$week> was specified and no C<$month> was specified.
+
+=item C<< week => $week >>
+
+C<$week> sets the week of the year if no C<$month> was specified. If no C<$wday> was specified, it will default to the C<1st> day of the week, i.e. C<Monday>. 
+
+=item C<< yday => $yday >>
+
+C<$yday> sets the day of the year if neither C<$month> or C<$week> was specified.
+
+=item C<< year => $year >>
+
+C<$year> specifies the year, and if no C<$month>, C<$week>, or C<$yday> is specified, the day will default to C<January 1st>.
+
+=item C<< tz => $tz >>
+
+C<$tz> specifies the timezone, and will default to C<UTC> if neither C<$tz> or C<$offset> is given.
+
+=item C<< offset => $offset >>
+
+C<$offset> specifies the offset from C<UTC> in minutes, and will default to C<0> if neither C<$tz> nor C<$offset> are given.
+
+=back
+
 =head2 localtime
 
   my $t = Time::C->localtime($epoch);
@@ -614,42 +782,62 @@ Creates a Time::C object for the current epoch in C<UTC>.
 =head2 from_string
 
   my $t = Time::C->from_string($str);
-  my $t = Time::C->from_string($str, $format);
-  my $t = Time::C->from_string($str, $format, $expected_tz);
+  my $t = Time::C->from_string($str, format => $format);
+  my $t = Time::C->from_string($str, format => $format, locale => $locale);
+  my $t = Time::C->from_string($str, format => $format, locale => $locale, strict => $strict);
+  my $t = Time::C->from_string($str, format => $format, locale => $locale, strict => $strict, tz => $tz);
 
-Creates a Time::C object for the specified C<$str>, using the optional C<$format> to parse it, and the optional C<$expected_tz> to set an unambigous timezone, if it matches the offset the parsing operation gave.
+Creates a Time::C object for the specified C<$str>, using the optional C<$format> to parse it, and the optional C<$tz> to set an unambigous timezone, if it matches the offset the parsing operation gave.
 
 =over
 
 =item C<$str>
 
-This is the string that will be parsed by either L<Time::Piece/strptime> or L<Time::Moment/from_string>.
+This is the string that will be parsed by either L<Time::P/strptime> or L<Time::Moment/from_string>.
 
-=item C<$format>
+=item C<< format => $format >>
 
-This is the format that L<Time::Piece/strptime> will be given, by default it is C<undef>. If it is not defined, L<Time::Moment/from_string> will be used instead.
+If specified, will be passed to L<Time::P/strptime> for parsing. Otherwise, L<Time::Moment/from_string> will be used.
 
-=item C<$expected_tz>
+=item C<< locale => $locale >>
 
-If the parsed time contains a zone or offset that parses, and the offset matches the C<$expected_tz> offset, C<$expected_tz> will be set as the timezone. If it doesn't match, a generic timezone matching the offset will be set, such as C<UTC> for an offset of C<0>. This variable will also default to C<UTC>.
+If C<strptime> is used for parsing, it will be given the specified C<$locale>. Defaults to C<C>.
+
+=item C<< strict => $strict >>
+
+If C<strptime> is used for parsing, it will be given the specified C<$strict>. Defaults to C<1>.
+
+=item C<< tz => $tz >>
+
+If there is no valid timezone specified in the format, but C<$tz> is given and matches the offset, then C<$tz> will be set as the timezone. If it doesn't match, and there was no valid timezone specified in the format, a generic timezone matching the offset will be set, such as C<UTC> for an offset of C<0>. This variable will also default to C<UTC>.
 
 =back
 
 =head2 strptime
 
   my $t = Time::C->strptime($str, $format);
+  my $t = Time::C->strptime($str, $format, locale => $locale);
+  my $t = Time::C->strptime($str, $format, locale => $locale, strict => $strict);
 
-Creates a Time::C object for the specified C<$str> using the C<$format> to parse it with L<Time::Piece/strptime>.
+Creates a Time::C object for the specified C<$str> using the C<$format> to parse it with L<Time::P/strptime>.
 
 =over
 
 =item C<$str>
 
-This is the string that will be parsed by L<Time::Piece/strptime>.
+This is the string that will be parsed by L<Time::P/strptime>.
 
 =item C<$format>
 
-This is the format that L<Time::Piece/strptime> will be given.
+This is the format that L<Time::P/strptime> will be given.
+
+=item C<< locale => $locale >>
+
+Gives the C<$locale> parameter to L<Time::P/strptime>. Defaults to C<C>.
+
+=item C<< strict => $strict >>
+
+Gives the C<$strict> parameter to L<Time::P/strptime>. Defaults to C<1>.
 
 =back
 
@@ -713,15 +901,46 @@ If the form C<< $t->tm($new_tm) >> is used, it likewise changes the current epoc
 =head2 string
 
   my $str = $t->string;
-  my $str = $t->string($format);
+  my $str = $t->string(format => $format);
+  my $str = $t->string(format => $format, locale => $locale);
   $t->string = $str;
-  $t->string($format) = $str;
+  $t->string(format => $format) = $str;
+  $t->string(format => $format, locale => $locale) = $str;
+  $t->string(format => $format, strict => $strict) = $str;
+  $t->string(format => $format, locale => $locale, strict => $strict) = $str;
 
-  $t = $t->string($format, $new_str);
+  $t = $t->string($new_str, format => $format);
+  $t = $t->string($new_str, format => $format, locale => $locale);
+  $t = $t->string($new_str, format => $format, strict => $strict);
+  $t = $t->string($new_str, format => $format, locale => $locale, strict => $strict);
 
-Renders the current time to a string using the optional strftime C<$format>. If the C<$format> is not given it defaults to C<undef>. When setting this value, it tries to parse the string using L<Time::Piece/strptime> with the C<$format> or L<Time::Moment/from_string> if no C<$format> was given. If the detected C<offset> matches the current C<tz>, that is kept, otherwise it will get changed to a generic C<tz> in the form of C<Etc/GMT+X> or C<+XX:XX>.
+Renders the current time to a string using the optional strftime C<$format>. If the C<$format> is not given it defaults to C<undef>. When setting this value, it tries to parse the string using L<Time::P/strptime> with the C<$format>, C<$locale>, and C<$strict> settings, or L<Time::Moment/from_string> if no C<$format> was given.
 
-If the form C<< $t->string($format, $new_str) >> is used, it likewise updates the epoch and timezone but returns the entire object.
+If the format specifies a timezone, it will be updated if it is valid. If not, it checks if the detected C<offset> matches the current C<tz>, and if so, the C<tz> is kept, otherwise it will get changed to a generic C<tz> in the form of C<Etc/GMT+X> or C<+XX:XX>.
+
+If the form C<< $t->string($new_str) >> is used, it likewise updates the epoch and timezone but returns the entire object.
+
+B<Note>: this will not always round-trip for any given C<$format> currently, as the implementations of L<Time::Piece/strftime> and L<Time::P/strptime> have some differences, especially where locales and timezones are concerned.
+
+=over
+
+=item C<$new_str>
+
+If specified, it will update the object by parsing the C<$new_str> with L<Time::P/strptime> if a C<$format> was passed, or L<Time::Moment/from_string> otherwise.
+
+=item C<< format => $format >>
+
+If specified, will be passed to L<Time::P/strptime> for parsing, or L<Time::Piece/strftime> for formatting.
+
+=item C<< locale => $locale >>
+
+If C<strptime> is used for parsing, it will be given the specified C<$locale>. Defaults to C<C>.
+
+=item C<< strict => $strict >>
+
+If C<strptime> is used for parsing, it will be given the specified C<$strict>. Defaults to C<1>.
+
+=back
 
 =head2 strftime
 
@@ -950,6 +1169,10 @@ Like C<Time::C> but for durations.
 
 If you need C<Time::C> times to recurr at regular intervals.
 
+=item L<Time::P>
+
+For parsing times from strings.
+
 =item L<Time::Moment>
 
 This implements most of the logic of this module.
